 /*-------------------------------------------------------------------*
  *    Name: CONTOUR SAS                                              *
  *   Title: IML macro to plot elliptical contours for X, Y data      *
  *     Ref: IML User's Guide, version 5 Edition                      *
  *-------------------------------------------------------------------*
  *  Author:  Michael Friendly            <friendly@YorkU.ca>         *
  * Created:  8 Jun 1988 12:33:21                                     *
  * Revised: 22 May 2000 09:48:55                                     *
  * Version:  2.3                                                     *
  *    Observations with missing data are deleted (including .A-.Z)   *
  *    Fixed gensym, for more than 8 groups                           *
  *    Added line= option, selection of mean & group labels           *
  *    Added CLASS= synonym for GROUP=, reset SYMBOL at end           *
  *    Added WEIGHT=                                                  *
  *    Changed default pvalue = 0.68 = 'standard ellipse'             *
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *                                                                   *
  *-------------------------------------------------------------------*/
 
%macro CONTOUR(
	data=_LAST_,           /* input data set                   */
	x=,                    /* X variable                       */
	y=,                    /* Y variable                       */
	weight=,
	group=,                /* Group variable (optional)        */
	class=,
	pvalue= .68,           /* Confidence coefficient (1-alpha) */
	line=5,                /* line style for ellipse           */
	annoadd=MEAN GPLABEL,  /* additional annotations           */
	std=STDERR,            /* error bar metric: STD or STDERR  */
	points=40,             /* points on each contour           */
	all=NO,                /* include contour for total sample?*/
	out=CONTOUR,           /* output data set                  */
	plot=YES,              /* plot the results?                */
	haxis=,              /* AXIS statement for horizontal axis */
	vaxis=,              /* AXIS statement for vertical axis   */
	i=none,                /* SYMBOL statement interpolate opt */
	interp=,               /* (synonym)                        */
	name=CONTOUR,          /* Name for graphic catalog entry   */
	colors=RED GREEN BLUE BLACK PURPLE YELLOW BROWN ORANGE,
	symbols=dot square circle + star -     plus   :      $     = );
 
%let all = %upcase(&all);
%let annoadd = %upcase(&annoadd);
%if &x=%str() or &y=%str() %then %do;
   %put CONTOUR: X= and Y= variables must be specified;
   %goto DONE;
   %end;

%if %length(&class)>0 %then %let group=&class;
%if %length(&interp)>0 %then %let i=&interp;
 
proc iml;
start ellipse(c, x, y, npoints, pvalues, formean)
	global (wt);
  /*----------------------------------------------------------------*
   |  Computes elliptical contours for a scatterplot                |
   |   C       returns the contours as consecutive pairs of columns |
   |   X,Y     coordinates of the points                            |
   |   NPOINTS scalar giving number of points around a contour      |
   |   PVALUES column vector of confidence coefficients             |
   |   FORMEAN 0=contours for observations, 1=contours for means    |
   *----------------------------------------------------------------*/
 
   xx = x||y;
   n  = nrow(x);
	if type(wt) ^='N' then wt = j(n,1);
   *-- Correct for the mean --;
   mean = (wt#xx)[+,]/sum(wt);
   xx = xx - mean @ j(n,1,1);
 
   *-- Find principal axes of ellipses --;
   xx = xx` * diag(wt) * xx / (sum(wt)-1);
   * print 'Variance-Covariance Matrix',xx;
   call eigen(v, e, xx);
 
   *-- Set contour levels --;
   c =  2*finv(pvalues,2,int(sum(wt))-1,0);
   if formean=1 then c = c / (sum(wt)-1) ;
   * print 'Contour values',pvalues c;
   a = sqrt(c*v[ 1 ] );
   b = sqrt(c*v[ 2 ] );
 
   *-- Parameterize the ellipse by angles around unit circle --;
   t = ( (1:npoints) - {1}) # atan(1)#8/(npoints-1);
   s = sin(t);
   t = cos(t);
   s = s` * a;
   t = t` * b;
 
   *-- Form contour points --;
   s = ( ( e*(shape(s,1)//shape(t,1) )) +
         mean` @ j(1,npoints*ncol(c),1) )` ;
   c = shape( s, npoints);
   *-- C returned as NCOL pairs of columns for contours--;
finish;
 
start dogroups(x, y, gp, pvalue);
   d  = design(gp);
   %if &all=YES %then %do;
      d = d || j(nrow(x),1,1);
   %end;
   do group = 1 to ncol(d);
      * print group;
      *-- select observations in each group;
      col = d[, group ];
      xg  = x[ loc(col), ];
      yg  = y[ loc(col), ];
      *-- Find ellipse boundary ;
      run ellipse(xyg,xg,yg,&points, pvalue, 0 );
      nr = nrow(xyg);
 
      *-- Output contour data for this group;
      cnames = { X Y PVALUE GP };
      do c=1 to ncol(pvalue);
         col=(2*c)-1 : 2*c ;
         xygp = xyg[,col] || j(nr,1,pvalue[c]) || j(nr,1,group);
         if group=1 & c=1
            then create contour from xygp [colname=cnames];
         append from xygp;
      end;
   end;
finish;
 
start nomiss(matrix, obsnames, keep);
   *-- Remove rows with any missing data from matrix and obsnames;
   *   (pass ' ' for obsnames if no row labels are present);
   miss = loc(matrix <=.Z);
   if type(miss)='U'
      then do;
		   keep=1:nrow(matrix); 
			return;           /* no missing data */
			end;
      else do;
         nr = nrow(matrix);
         nc = ncol(matrix);
         rows= 1+floor((miss-1)/nc);
         rows= unique(rows);
         keep=remove(1:nrow(matrix),rows);
         deleted = nr - ncol(keep);
         matrix = matrix[keep,];
         reset noname;
         print 'Warning:' deleted 'row(s) with missing data have been removed.';
         reset name;
         if obsnames ^={' '}
            then do;
               obs = obsnames[rows];
               obs = shape(obs,1,nrow(obs)#ncol(obs));
               if type(obs)='N' then obs=char(obs,3,0);
               obsnames=obsnames[keep];
            *  print rows[c=obs], obs;
            end;
         end;
finish;

*-- Get input data: X, Y, GP;
   use &data;
   read all var {&x}     into x [colname=lx];
   read all var {&y}     into y [colname=ly];
	xy = x || y;
	run nomiss(xy, ' ', kept);
%if %length(&weight) > 0 %then %do;
   read all var {&weight} into wt [colname=lg] ;
	if ncol(keep) ^= nrow(xy) then wt = wt[kept];
   %end;
%if %length(&group) > 0 %then %do;
   read all var {&group} into gp [colname=lg] ;
	if ncol(keep) ^= nrow(xy) then gp = gp[kept];
   %end;
%else %do;
   gp = j(nrow(x),1,1);
   %end;
	x = xy[,1];
	y = xy[,2];
   close &data;
 
*-- Find contours for each group;
   run dogroups(x, y, gp, { &pvalue} );
quit;
 
 /*-----------------------------------*
  |  Plot the contours using ANNOTATE |
  *-----------------------------------*/
data &out;
   set contour end=eof;
   by gp pvalue notsorted;
	retain g 0;
	drop g;
	if first.gp then g+1;
   length function color $8;
   xsys='2'; ysys='2';
   if first.pvalue then function='POLY';
                   else function='POLYCONT';
   color=scan("&colors",gp);
   line = &line;
   if eof then call symput('NGROUP',put(g,best.));
run;
 /*----------------------------*
  |  Crosses at Mean +- StdErr |
  *----------------------------*/
%if &annoadd ^= NONE %then %do;
proc summary data=&data nway;
   class &group;
   var &x &y;
   output out=sumry mean=mx my &std=sx sy;
run;
data bars;
   set sumry end=eof;
   %if &group ^= %str() %then %str(by &group;);
   length function color $8;
   retain g 1;
   drop _freq_ _type_ mx my sx sy g;
   xsys='2'; ysys='2';
 
   color=scan("&colors",g);
	%if %index(&annoadd,MEAN) > 0 %then %do;
   line=3;
   x = mx-sx; y=my;      function='MOVE'; output;
   x = mx+sx;            function='DRAW'; output;
   x = mx   ; y=my-sy;   function='MOVE'; output;
              y=my+sy;   function='DRAW'; output;
	%end;
 
 *-- Write group label (convert numeric &group to character);
   %if &group ^= %str() and %index(&annoadd,GPLABEL) > 0 %then %do;
      length text $16;
      text = left(&group);
      position='3';
      size = 1.4;
      x = mx+.2*sx   ; y=my+.2*sy;
      function='LABEL'; output;
   %end;
   %if &group ^= %str() %then %do;
      if last.&group then g+1;
   %end;
run;
 
data &out;
   set &out bars;
%end; /* annoadd ^= NONE */
 
%if %upcase(&plot)=YES %then %do;

   %gensym(n=&ngroup, h=1.2, i=&i, colors=&colors, symbols=&symbols );

   %if %length(&haxis)=0 %then %do;
      axis2 offset=(3) value=(h=1.5) label=(h=1.5);
      %let haxis=axis2;
      %end;
   %if %length(&vaxis)=0 %then %do;
      axis1 offset=(3) value=(h=1.5) label=(h=1.5 a=90);
      %let vaxis=axis1;
      %end;

	%if &group = %str() %then %let group=1;
   proc gplot data=&data &GOUT ;
      plot &y * &x = &group /
           annotate=&out
           nolegend frame
           vaxis=&vaxis vminor=0
           haxis=&haxis hminor=0
           name="&name";
      run; quit;
goptions reset=symbol;
%end;
%done: ;
%mend contour;
 
 /*----------------------------------------------------*
  |  Macro to generate SYMBOL statement for each GROUP |
  *----------------------------------------------------*/
%macro gensym(n=1, h=1.5, i=none,
              symbols=%str(- + : $ = X _ Y),
              colors=BLACK RED GREEN BLUE BROWN YELLOW ORANGE PURPLE);
    %*--  if more than 8 groups, symbols and colors are recycled;
  %local chr col k;
  %do k=1 %to &n ;
     %if %length(%scan(&symbols, &k, %str( ))) = 0 
	      %then %let symbols = &symbols &symbols;
     %if %length(%scan(&colors, &k, %str( ))) = 0 
	      %then %let colors = &colors &colors;
     %let chr =%scan(&symbols, &k,%str( ));
     %let col =%scan(&colors, &k, %str( ));
     symbol&k h=&h v=&chr c=&col i=&i;
  %end;
%mend gensym;
