 /*-------------------------------------------------------------------*
  *    Name: scatmat.sas                                              *
  *   Title: scatterplot matrix - all pairwise plots for n variables  *
  *     Doc: http://www.math.yorku.ca/SCS/sssg/scatmat.html           *
  *   Usage: %scatmat(data=, var=, group=);                           *
  *-------------------------------------------------------------------*
  *  Author:  Michael Friendly            <friendly@yorku.ca>         *
  * Created:  4 Oct 1989 11:07:50                                     *
  * Revised:  5 Mar 1999 16:37:24                                     *
  * Version:  1.5                                                     *
  *    - Added anno=ELLIPSE to draw data ellipse on off-diag plots    *
  *    - Added anno=LOWESS to draw lowess curve on off-diag plots     *
  *    - Added anno=BOX to draw boxplots on diag panels               *
  *    - Added PLOTOPT= for additional plot options                   *
  *    - Cleaned up temporary gseg library                            *
  *    - Added NAMES= to use different var labels than their names    *
  *                                                                   *
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *                                                                   *
  * Requires: %gdispla %lowess %contour %boxaxis                      *
  *-------------------------------------------------------------------*/
 /*=
=Dependencies:
 
 Depending on options selected, the scatmat macro calls several
 other macros not included here.  It is assumed these are stored
 in an autocall library.  If not, you'll have to %include each one
 you use.
 
 %gdispla  - device-independent DISPLAY/NODISPLAY control [always]
 %lowess   - smoothed lowess curves        [ANNO incl. LOWESS]
 %contour  - data ellipses                 [ANNO incl. ELLIPSE]
 %boxaxis  - boxplot for diagonal panels   [ANNO incl. BOX]
 
 These are all available by ftp://hotspur.psych.yorku.ca/pub/sas
 (though in different subdirectories).
 
 Warning: the %lowess macro is quite slow for large data sets,
 particularly when run for each pair of variables.  In SAS V7+
 the %lowess macro uses proc loess, which is much faster.
  
 =*/

%macro scatmat(
        data =_LAST_,          /* data set to be plotted             */
        var  =_NUMERIC_,       /* variables to be plotted - can be   */
                               /* a list or X1-X4 or VARA--VARB      */
		  names=,                /* Alternative variable names         */
        group=,                /* grouping variable (plot symbol)    */
        interp=none,           /* plot interpolation method          */
        hsym=,                 /* height of plot symbols             */
		  htitle=,               /* height of variable name titles     */
		  plotopt=,              /* additional plot options            */
        symbols=%str(circle square + : $ = X _ Y),
        colors=BLACK RED GREEN BLUE BROWN YELLOW ORANGE PURPLE,
        anno=NONE,             /* annotate diag or off-diag plot     */
		  gtemp=gtemp,           /* temporary graphics catalog         */
		  kill=Y,                /* delete grtemp when done            */
        gout=GSEG);            /* graphic catalog for plot matrix    */
 
 options nonotes;
%let anno=%upcase(&anno);
%if %upcase(&data)=_LAST_ %then %let data = &syslast;

%*-- For Session Manager, need to start a new graphic catalog;
%*-- Use a temporary graphics catalog to create the individual plots;
%*let gtemp = gtemp;
 
*-- Parse variables list;
 data _null_;
 set &data (obs=1) nobs=nobs;
 	call symput('nobs', trim(left(put(nobs,8.))) );
    %if %index(&var,-) > 0 or "&var"="_NUMERIC_" %then %do;
 
       %* find the number of variables in the list and
         convert shorthand variable list to long form;
     length _vname_ $ 8 _vlist_ $ 200;
     array _xx_ &var;
     _vname_ = ' ';
     do over _xx_;
        call vname(_xx_,_vname_);
        if _vname_ ne "&group" then do;
           nvar + 1;
	        _vlist_ = trim(_vlist_)|| ' ' || trim(_vname_);
        end;
     end;
     call symput( 'VAR', trim(_vlist_) );
   %end;
   %else %do;
     * find the number of variables in the list;
     nvar = n(of &var) + nmiss(of &var);
   %end;
   call symput('NVAR',trim(left(put(nvar,2.))));
     * default symbol height, if hsym not specified;
   ht = scan('1.4 2.3 3 3.7 4.2 4.5 5 5.3 5.4 5.5',nvar,' ');
   call symput('HT',trim(left(put(ht,3.1))));
 RUN;
%if &nvar < 2 or &nvar > 10 %then %do;
   %put Cannot do a scatterplot matrix for &nvar variables ;
   %goto DONE;
   %end;
 
 /*----------------------------------------------------*
  | Determine grouping variable and plotting symbol(s) |
  *----------------------------------------------------*/
%if &group = %str() %then %do;
   %let NGROUPS=1;
   %let plotsym=1;      /* SYMBOL for data panels  */
   %let plotnam=2;      /* for variable name panel */
   %end;
%else %do;
   %let plotsym=&group;
   *-- How many levels of group variable? --;
   proc freq data = &data;
      tables &group / noprint out=_DATA_;
   data _null_;
      set _LAST_(obs=1) nobs=ngroups;
      call symput( 'NGROUPS', put(ngroups,3.) );
    run;
    %let plotnam=%eval(&ngroups+1);
%end;
 
%if &hsym = %str() %then %let h=&ht;
                   %else %let h=&hsym;
%gensym(n=&ngroups, h=&h, i=&interp, symbols=&symbols, colors=&colors);
SYMBOL&plotnam v=none i=none;

%gdispla(OFF);
 
title h=0.01 ' ';
%let annoopt=;
%let plotnum=0;    * number of plots made;
%let replay = ;    * replay list;

%if %length(&htitle)=0 %then %let htitle=2*&nvar;
 
%do i = 1 %to &nvar;                   /* rows */
   %let vi = %scan(&var , &i );
	%let ni=;
	%if %length(&names)>0
		%then %let ni = %scan(&names , &i );
   proc means noprint data=&data;
      var &vi;
      output out=minmax min=min max=max;
 
   %do j = 1 %to &nvar;                /* cols */
      %let vj = %scan(&var , &j );
      %let plotnum = %eval(&plotnum+1);
      %let replay  = &replay &plotnum:&plotnum ;
      %put plot &plotnum: &vi vs. &vj ;
 
      %if &i = &j %then %do;           /*---- diagonal panel ----*/
         data title;
            length text function $8;
            set minmax;
            xsys = '1'; ysys = '1';
            x = 50; y = 50;
            text = "&vi";
				%if %length(&ni)>0 %then %str(text = "&ni";);
            size = &htitle;
            function = 'LABEL';  output;
 
            x = 6; y = 6; position = '6';
            text = left(put(min, best6.));
            size = &nvar;
            output;
 
            x = 95; y = 95; position = '4';
            text = trim(put(max, best6.));
            size = &nvar;
            output;
 
 			%if %index(&anno,BOX) %then %do;
			%boxaxis(data=&data, var=&vi, baxis=y, oaxis=x, out=_anno_,
				pos=50, boxwidth=10, hsym=&h);
			data title;
				set title _anno_;
			%end;
         proc gplot data = &data gout=&gtemp;
            plot &vi * &vi = &plotnam
            / frame anno=title vaxis=axis1 haxis=axis1
				  name="scat&i" des="SCATMAT title &vi";
         axis1 label=none value=none major=none
               minor=none offset=(2);
         run; quit;
      %end;
 
      %else %do;                     /*---- off-diagonal panel ----*/

			%let inanno=;
			%if %index(&anno,ELLIPSE) %then %do;
			%contour(data=&data, x=&vj, y=&vi, group=&group, annoadd=NONE,
					out=_anno_, colors=&colors, plot=NO);
			%let annoopt=anno=_anno_;
			%let inanno=_anno_;
			%end;
			
			%if %index(&anno,LOWESS) %then %do;
			%lowess(data=&data, x=&vj, y=&vi, p=1, outanno=_anno2_,
					colors=&colors, plot=NO, silent=YES, step=0, in=&inanno);
			%let annoopt=anno=_anno2_;
			%let inanno=_anno2_;
			/*
			%if %index(&anno,ELLIPSE) %then %do;
				data _anno_;
					set _anno_ _anno2_;
				%let annoopt=anno=_anno_;
				%end;
			%else %do;
				%let annoopt=anno=_anno2_;
				%end;
			*/
			%end;
			
         proc gplot data = &data gout=&gtemp;
            plot &vi * &vj = &plotsym / &annoopt
              frame nolegend vaxis=axis1 haxis=axis1 &plotopt
				  name="scat&i.&j" des="SCATMAT plot &vi, &vj";
         axis1 label=none value=none major=none minor=none offset=(2);
         run; quit;
      %end;
 
   %end; /* cols */
%end;    /* rows */
 
%gdispla(ON);
 
%macro tdef(nv, size, shift );
%* ---------------------------------------------------------------;
%* Generate a TDEF statement for a scatterplot matrix             ;
%* Start with (1,1) panel in upper left, and copy it across & down;
%* ---------------------------------------------------------------;
%local i j panl panl1 lx ly;
 
   TDEF scat&nv DES="scatterplot matrix &nv x &nv"
   %let panl=0;
   %let lx = &size;
   %let ly = %eval(100-&size);
   %do i = 1 %to &nv;
   %do j = 1 %to &nv;
       %let panl  = %eval(&panl + 1);
       %if &j=1 %then
          %do;
             %if &i=1 %then %do;      %* (1,1) panel;
               &panl/
                ULX=0  ULY=100   URX=&lx URY=100
                LLX=0  LLY=&ly   LRX=&lx LRY=&ly
                %end;
             %else
                %do;                  %* (i,1) panel;
                   %let panl1 = %eval(&panl - &nv );
               &panl/ copy= &panl1 xlatey= -&shift
                %end;
          %end;
       %else
          %do;
               %let panl1 = %eval(&panl - 1);
               &panl/ copy= &panl1 xlatex= &shift
          %end;
   %end;
   %end;
     %str(;);      %* end the TDEF statement;
%mend;
 
proc greplay igout=&gtemp
              gout=&gout  nofs
             template=scat&nvar
             tc=templt ;
%if &nvar = 2 %then %do;
  TDEF scat2 DES="scatterplot matrix 2x2"
           1/ ULX=0  ULY=100   URX=52  URY=100
              LLX=0  LLY=52    LRX=52  LRY=52
           2/ copy=1 XLATEX= 48       /*  Panels are numbered: */
           3/ copy=1 XLATEY=-48                /*    1   2     */
           4/ copy=3 XLATEX= 48;               /*    3   4     */
%end;
 
%if &nvar = 3 %then  %TDEF(&nvar,33,33);
%if &nvar = 4 %then  %TDEF(&nvar,25,25);
%if &nvar = 5 %then  %TDEF(&nvar,20,20);
%if &nvar = 6 %then  %TDEF(&nvar,16,16);
%if &nvar = 7 %then  %TDEF(&nvar,14,14);
%if &nvar = 8 %then  %TDEF(&nvar,13,12);
%if &nvar = 9 %then  %TDEF(&nvar,12,11);
%if &nvar =10 %then  %TDEF(&nvar,10,10);
 
   TREPLAY &replay;
run;
%if &kill=Y %then %do;
  proc catalog kill catalog=&gtemp et=grseg;
run; quit;
%end;
%DONE:
  options notes;
  goptions reset=symbol;
%mend;
 
 
 /*----------------------------------------------------*
  |  Macro to generate SYMBOL statement for each GROUP |
  *----------------------------------------------------*/
%macro gensym(n=1, h=1.5, i=none,
              symbols=%str(- + : $ = X _ Y),
              colors=BLACK RED GREEN BLUE BROWN YELLOW ORANGE PURPLE);
    %*--  if more than 8 groups, symbols and colors are recycled;
  %local chr col k;
  %do k=1 %to &n ;
     %if %length(%scan(&symbols, &k, %str( ))) = 0 
	      %then %let symbols = &symbols &symbols;
     %if %length(%scan(&colors, &k, %str( ))) = 0 
	      %then %let colors = &colors &colors;
     %let chr =%scan(&symbols, &k,%str( ));
     %let col =%scan(&colors, &k, %str( ));
     symbol&k h=&h v=&chr c=&col i=&i;
  %end;
%mend gensym;
